# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 3
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

bl_info = {
    "name": "Export particles as matrices",
    "description": "exports the particles system associated with the current object and exports it to a text file as transformation matrixes.",
    "author": "Stephen Boyd",
    "version": (1, 0),
    "blender": (2, 63, 0),
    "location": "View3D > Tool shelf",
    "warning": "", 
    "wiki_url": "",
    "tracker_url": "",
    "category": "Import-Export"}


import bpy
import mathutils
import math

class ExportMatrices(bpy.types.Operator):
    bl_idname = "export.matrices"
    bl_label = "Export Matrices"

    filepath = bpy.props.StringProperty(subtype="FILE_PATH")

    @classmethod
    def poll(cls, context):
        return context.object is not None

    def execute(self, context):
        file = open(self.filepath, 'w')
       
        obj = bpy.context.object
        ps = obj.particle_systems.active

        for parts in ps.particles:
            alv = parts.is_visible
            if alv:
               locx = parts.location[0]
               locy = parts.location[1]
               locz = parts.location[2]
    
# Convert Quaternion to Euler
               quat = parts.rotation
               eu = quat.to_euler('XYZ')
    
# Now build the matrix
               sinx = math.sin(eu[0])
               cosx = math.cos(eu[0])
               siny = math.sin(eu[1])
               cosy = math.cos(eu[1])
               sinz = math.sin(eu[2])
               cosz = math.cos(eu[2])
    
               cell1 = int(1000 * cosy * cosz) / 1000
               cell2 = int(1000 * cosy * sinz) / 1000
               cell3 = int(1000 *  siny) / 1000
               cell4 = int(1000 * locx) / 1000
               cell5 = int(1000 * (cosz * (0 - sinx) * siny - cosx * sinz)) / 1000
               cell6 = int(1000 * ((0 - sinx) * siny * sinz + cosx * cosz)) / 1000
               cell7 = int(1000 * cosy * sinx ) / 1000
               cell8 = int(1000 * locy) / 1000
               cell9 = int(1000 * (sinx * sinz + cosx * cosz * siny)) / 1000
               cell10= int(1000 * ((0 - cosx) * siny * sinz - sinx * cosz)) / 1000
               cell11= int(1000 * cosx * cosy) / 1000
               cell12= int(1000 * locz) / 1000
            
               file.write(str(cell1) + "," + str(cell2) + "," + str(cell3) + "," + str(cell4) + "," + str(cell5) + "," + str(cell6) + "," + str(cell7) + "," + str(cell8) + "," + str(cell9) + "," + str(cell10) + "," + str(cell11) + "," + str(cell12) + "\n")
          
       
        return {'FINISHED'}
    
    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

def menu_func(self, context):
    self.layout.operator_context = 'INVOKE_DEFAULT'
    self.layout.operator(ExportMatrices.bl_idname, text="Export Matrices")

def register():
    bpy.utils.register_class(ExportMatrices)


def unregister():
    bpy.utils.unregister_class(ExportMatrices)
